 /* AddMiniApp.java
 * @author Mike Ross
 * 
 * The screen for adding MiniApps.  Uses a grid layout. Selecting a MiniApp
 * icon will go to that MiniApp's creation activity, where the user can
 * edit its contents (e.g. Zipcode, URL, notes).
 * 
 */

package edu.berkeley.cs160.alarmbuddy;

/* ===== MYALARMS.JAVA =====
 * 
 * Author: Mike Ross
 * 
 * This activity represents the the "Add MiniApps" screen that users see
 * upon selecting "add mini apps" when editing an alarm.
 * 
 * The Screen offers a list of applications they can add to their alarm.
 * They can add multiple instances of the same application if they want, e.g.
 * multiple Browser MiniApps for different webpages, multiple Notepad Apps for
 * different notes, etc.  
 * 
 * 1. Activity Lifecycle
 * 		a. onCreate
 * 		b. onResume
 * 		c. onFreeze
 * 		d. onPause
 * 2. Other Functions
 * 
 *  ===================== 
 */

import android.app.Activity;
import android.content.Intent;
import android.database.Cursor;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.AdapterView;
import android.widget.Button;
import android.widget.GridView;
import android.widget.SimpleCursorAdapter;

public class AddMiniApp extends Activity {

	public static final int ACTIVITY_ADD_MINIAPP = 0;
	private static final String KEY_UNIQUE_ALARM_ID = "unique_alarm_id";
	
	private InstalledMiniAppDbAdapter mInstalledMiniAppsDb;
	private MiniAppDb mMiniAppDb;
	private Cursor mMiniAppsCursor;
	private long mAlarmId;
	

	
	static final MiniAppActivity mMiniAppActivities[] = {
		//new Browser(null),
		new Weather(),
		new ToDoMiniAppWrapper(),
		new WeatherReal(),
	};	
	
	GridView mGrid;

	/* 1. Activity Lifecycle */

	/** 1a. onCreate : Called when the activity is first created. */
	@Override
	public void onCreate(Bundle icicle) {
		
		super.onCreate(icicle);
		
		Bundle extras = getIntent().getExtras(); //the bundle with miniAppId info
		assert extras != null;
		
		setContentView(R.layout.add_mini_app);

		mAlarmId = extras.getLong(MiniAppDb.KEY_ALARM_ID);
		mMiniAppDb = new MiniAppDb(this, mAlarmId);
		mMiniAppDb.open();
		
		mInstalledMiniAppsDb = new InstalledMiniAppDbAdapter(this);
		mInstalledMiniAppsDb.open();
		installMiniApps();
		
		mGrid = (GridView) findViewById(R.id.miniAppsGrid);

		fillData();
		
		mGrid.setSelection(0);
		
		final Activity context = this;
		
		// fire up a new MiniApp screen if confirm button is hit.
		mGrid.setOnItemClickListener(new AdapterView.OnItemClickListener() {
			/** Opens the appropriate MiniApp Creation activity **/
			public void onItemClick(AdapterView parent, View v, int position, long id) {
				MiniAppActivity ma = mMiniAppActivities[(int)id];
				Class<MiniAppActivity> maClass = ma.getActivityClass();
				Intent i = new Intent(context, maClass);
				i.putExtra(MiniAppDb.KEY_ALARM_ID, mAlarmId);
				i.setAction("EDIT");
				
				// create the entry in the database
				// this should be deleted in the event of a cancel.
				mMiniAppDb.updateInstance(ma.getNiceName());
				startSubActivity(i, ACTIVITY_ADD_MINIAPP);
			}
        });
		
		Button mCancelButton = (Button) findViewById(R.id.amaCancel);
        Button mConfirmButton = (Button) findViewById(R.id.amaConfirm);
        
        mConfirmButton.setOnClickListener(new View.OnClickListener() {
        	public void onClick(View v) {
        		onConfirmButtonClick();
            }
        });
        
        mCancelButton.setOnClickListener(new View.OnClickListener() {
        	public void onClick(View v) {
        		onCancelButtonClick();
            }
        });
		
	}


	/**
	 * Read the available MiniApps from the DataBase.
	 */
	private void fillData() {
		// Get all of the rows from the database and create the item list
		mMiniAppsCursor = mInstalledMiniAppsDb.fetchAllMiniApps();
		startManagingCursor(mMiniAppsCursor);

		// Create an array to specify the fields we want to display in the list (only TITLE)
		String[] from = new String[]{
		            InstalledMiniAppDbAdapter.KEY_NICE_NAME,
					InstalledMiniAppDbAdapter.KEY_ICON,
					};

		// and an array of the fields we want to bind those fields to (in this case just text1)
		int[] to = new int[] {
					R.id.mini_app_title,
					R.id.mini_app_background,
					};
		int count = mMiniAppsCursor.count();
		for(mMiniAppsCursor.first(); mMiniAppsCursor.position() < count; mMiniAppsCursor.next()) {
			String icon = mMiniAppsCursor.getString(mMiniAppsCursor.getColumnIndex(InstalledMiniAppDbAdapter.KEY_ICON));
		}
		mMiniAppsCursor.first();		
		// Now create a simple cursor adapter and set it to display
		//SimpleCursorAdapter miniApps = 
		//	new SimpleCursorAdapter(this, R.layout.mini_app_button,mMiniAppsCursor, from, to);
		//setListAdapter(notes);
		MiniAppAdapter maAdapter = new MiniAppAdapter(this, mMiniAppActivities);		
		mGrid.setAdapter(maAdapter);
	}
	
	/** 1b. onResume : Called when the activity will start interacting 
	 * with the user. */
	@Override
	public void onResume() {
		super.onResume();
	}

	/** 1c. onFreeze : Allows you to save away your current state, 
	 * when your activity is being paused and another one resuming 
	 * to interact with the user. */
	@Override
	public void onFreeze(Bundle outState) {
	}

	/** 1d. onPause : Called when the system is about to start resuming 
	 * a previous activity.  */
	@Override
	public void onPause() {
		super.onPause();
	}

	/** When the miniApp add activity is done, this method will be run to
	 * process the updated alarm.
	 * 
	 */
	@Override
	protected void onActivityResult(int requestCode, int resultCode, String data, Bundle extras) {
		super.onActivityResult(requestCode, resultCode, data, extras);
		if (resultCode == Activity.RESULT_OK) {
			// do stuff
			if (requestCode == ACTIVITY_ADD_MINIAPP) {
				// add miniapp table name and rowId to this alarm's MiniApp table.
				//	long instanceId = extras.getLong(MiniAppDb.KEY_INSTANCE_ID);
				//	String miniAppName = extras.getString(MiniAppDb.KEY_MINIAPP_NAME);
				// mMiniAppDb.createInstance(miniAppName, instanceId);
				
			}
		}
	}
	
	
	/**
	 * Install all MiniApps that aren't currently listed as installed.
	 */
	private void installMiniApps() {
		
		int length = mMiniAppActivities.length;
		for(int i=0; i < length; i++) {
			MiniAppActivity m = mMiniAppActivities[i];
			String name = m.getNiceName();
			if(!mInstalledMiniAppsDb.miniAppIsInstalled(name)) {
				mInstalledMiniAppsDb.installMiniApp(m);
			}
		}
	}
	
	
    public void onConfirmButtonClick() {
    	setResult(RESULT_OK, null, null);
        finish();
    }
    
    public void onCancelButtonClick() {
    	setResult(RESULT_CANCELED, null, null);
        finish();
    }
	
}