/**
 * A pair of classes to facilitate viewing miniapps
 */

package edu.berkeley.cs160.alarmbuddy;

import android.content.Context;
import android.database.Cursor;
import android.graphics.Color;
import android.text.Layout.Alignment;
import android.text.style.AlignmentSpan;
import android.util.Log;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.LinearLayout;
import android.widget.TextView;

class AlarmAdapterView extends LinearLayout {
		private Context mContext;
		private boolean mActive;
		private boolean mIsTimer;
        public AlarmAdapterView(Context context,
        						long alarmId,
        						String title,
        						int time,
        						int active,
        						String recurrence,
        						int customRecurrence,
        						String tone,
        						int isTimer) {
            super(context);
            mContext = context;
            mActive = !(active==0);
            mIsTimer = 1 == isTimer; 
            this.setOrientation(VERTICAL);

            // create image.
            //LinearLayout.LayoutParams iconParams = 
            //    new LinearLayout.LayoutParams(50, 50);
            //iconParams.setMargins(1,1,1,1);
            
            // ImageView iconButton = new ImageView(context);
            // int iconAddress = m.getIconAddress();
            // iconButton.setImageResource(iconAddress);
            // addView(iconButton, iconParams);
            
            TextView titleControl = createTextControl(title);
            TextView timeControl;
            TextView recurrenceControl;
            if(mIsTimer) {
            	timeControl = createTextControl(timeToTimerString(time));
            	recurrenceControl = createTextControl("Timer");
            	this.setBackground(mActive ? 
            			R.drawable.bg_green_active : R.drawable.bg_green_inactive);
        	} else {
            	timeControl = createTextControl(timeToAlarmString(time));
                if(recurrence == "none") {
                	recurrence = "no recurrence";
                }
            	recurrenceControl = createTextControl(recurrence);
            	this.setBackground(mActive ? 
            			R.drawable.bg_blue_active : R.drawable.bg_blue_inactive);
            }
            LinearLayout.LayoutParams layoutParams = 
            	new LinearLayout.LayoutParams(LayoutParams.FILL_PARENT,
            			64);
            this.setLayoutParams(layoutParams);
        }
        
        private TextView createTextControl(String value) {
        	LinearLayout.LayoutParams textControlParams = 
                new LinearLayout.LayoutParams(
                		LayoutParams.WRAP_CONTENT, LayoutParams.WRAP_CONTENT);
            textControlParams.setMargins(1, 1, 1, 1);

            TextView textControl = new TextView(mContext);
            textControl.setGravity(0x01);
            Alignment align = Alignment.ALIGN_CENTER;
            textControl.setAlignment(align);
            textControl.setText( value );
            textControl.setTextSize( 14f );
            textControl.setTextColor(mActive ? Color.WHITE : Color.LTGRAY);
            addView(textControl, textControlParams);
            return textControl;
        }
        
        /**
         * take time in total minutes, return am/pm clock time in a String 
         * @param time
         * @return
         */
        private String timeToAlarmString(int time) {
	        String ampm;
	        int hours = time / 60;
	    	if(hours > 12) {
	    		hours -= 12;
	    		ampm = "p.m.";
	    	} else {
	    		ampm = "a.m.";
	    		if(time == 0) {
	    			hours = 12;
	    		}
	    	}
	    	String result = "";
	    	int minutes = time%60;
	    	String minString = "";
	    	minString += (minutes < 10) ?  ("0"+ minutes) : minutes;
	    	result += hours + ":" + minString + " " + ampm;  
	    	return result;
        }
        
        /**
         * take time in total minutes, return am/pm clock time in a String 
         * @param time
         * @return
         */
        private String timeToTimerString(int time) {
	        String ampm;
	        int hours = time / 60;
	        String hoursString = (hours < 10) ? "0" + hours : hours + ""; 
	    	int minutes = time%60;
	    	String minString = (minutes < 10) ?  ("0"+ minutes) : minutes +"";
	    	String result = hoursString + ":" + minString;  
	    	return result;
        }
}

class AlarmAdapter extends BaseAdapter {

    private Context mContext;
    private Cursor mCursor;

    public AlarmAdapter(Context context, Cursor c) { 
        this.mContext = context;
        this.mCursor = c;
    }

    public int getCount() {                        
        return mCursor.count();
    }

    public Object getItem(int position) {     
        if (mCursor.moveTo(position))
        	return mCursor;
        return null;
    }

    public long getItemId(int position) {
    	boolean success = mCursor.moveTo(position);
    	if(success)
    		return mCursor.getLong(mCursor.getColumnIndex(AlarmsDbAdapter.KEY_ROWID));
    	return -1;
    }

    public View getView(int position, View convertView, ViewGroup parent) { 
    	Cursor alarm = mCursor;
    	mCursor.moveTo(position);
    	long alarmId = alarm.getLong(alarm.getColumnIndex(AlarmsDbAdapter.KEY_ROWID));
		String title = alarm.getString(alarm.getColumnIndex(AlarmsDbAdapter.KEY_TITLE));
		int time = alarm.getInt(alarm.getColumnIndex(AlarmsDbAdapter.KEY_TIME));
		int active = alarm.getInt(alarm.getColumnIndex(AlarmsDbAdapter.KEY_ACTIVE));
		String recurrence = alarm.getString(alarm.getColumnIndex(AlarmsDbAdapter.KEY_RECURRENCE));
		int customRecurrence =  alarm.getInt(alarm.getColumnIndex(AlarmsDbAdapter.KEY_CUSTOM_RECURRENCE));
		String tone = alarm.getString(alarm.getColumnIndex(AlarmsDbAdapter.KEY_TONE));
		int isTimer = alarm.getInt(alarm.getColumnIndex(AlarmsDbAdapter.KEY_IS_TIMER));

        return new AlarmAdapterView(this.mContext,
        	alarmId, title, time, active, recurrence, customRecurrence, tone, isTimer);
    }
}
