package edu.berkeley.cs160.alarmbuddy;

import java.io.FileNotFoundException;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.SQLException;
import android.database.sqlite.SQLiteDatabase;

/**
 * Browser.java
 * @author Mike Ross
 *
 * Alarm Browser Mini Application
 *  
 * This will eventually provide an example for how to create a MiniApp.
 * For now, it's lacking methods.
 */
public class BrowserDb {

	private static final String KEY_ROWID = "_id";
	private static final String KEY_URL = "url";
	private static final String DATABASE_NAME = "data";
	private static final String DATABASE_TABLE = "browser_mini_app";
	private static final int	DATABASE_VERSION = 2;
	
	private static final String NICE_NAME = "Browser";
	private static SQLiteDatabase mDb;
	private Context mCtx;
	
	/* to be set per class */
	protected String getDbCreateString() {
		return "create table if not exists " + DATABASE_TABLE + " ("
		+ KEY_ROWID + " integer primary key autoincrement, "
		+ KEY_URL + " text not null "
		+ ");";
	}
	
	/**
     * Constructor - takes the context to allow the database to be opened/created
     * @param ctx the Context within which to work
     */
    public BrowserDb(Context ctx) {
        this.mCtx = ctx;
    }
    
    public void close() {
        mDb.close();
    }
    
    /**
     * Open the MiniApp's database. If it cannot be opened, try to create a new instance of
     * the database. If it cannot be created, throw an exception to signal the failure
     * @return this (self reference, allowing this to be chained in an initialization call)
     * @throws SQLException if the database could be neither opened or created
     */
    protected BrowserDb open() throws SQLException {
        try { 
            mDb = mCtx.openDatabase(DATABASE_NAME, null);
            
            mDb.execSQL(getDbCreateString());
        } catch (FileNotFoundException e) {
            try {
                mDb =
                    mCtx.createDatabase(DATABASE_NAME, DATABASE_VERSION, 0,
                        null);
                mDb.execSQL(getDbCreateString());
            } catch (FileNotFoundException e1) {
                throw new SQLException("Could not create database");
            }
        }
        return this;
        
    }
    
    /**
     * Create a new Browser Mini App row using the title and body provided.
     *  If the row is successfully created, return the new rowId,
     *  otherwise return a -1 to indicate failure.
     * @param title the title of the note
     * @param body the body of the note
     * @return rowId or -1 if failed
     */
    public long createInstance(String zipCode) {
    	ContentValues values = new ContentValues();
    	values.put(KEY_URL, zipCode);
    	return mDb.insert(DATABASE_TABLE, null, values);
    }
    
    public Cursor fetchMiniAppCursor(long rowId) throws SQLException {
    	Cursor c = mDb.query(DATABASE_TABLE, null, KEY_ROWID + "=" + rowId,
    			null, null, null, null);
    	if ((c.count() == 0) || !c.first()) {
    		throw new SQLException("No Browser Instance with row ID: " + rowId);
    	}
    	return c;
    }
    
    public String getZipCode(long rowId) {
    	Cursor c = fetchMiniAppCursor(rowId);
    	return c.getString(c.getColumnIndex(KEY_URL));
    }
    
    
    
    public int getIcon() {
    	return R.drawable.test_icon;
    }
    public String getInstanceTableName() {
    	return DATABASE_TABLE;
    }
    public String getNiceName() {
    	return NICE_NAME;
    }
}

