package edu.berkeley.cs160.alarmbuddy;

/* ===== EditTimer.JAVA =====
 *  Author: 
 *  
 *  
 *  ===================== 
 */



import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.database.Cursor;
import android.graphics.Matrix;
import android.graphics.drawable.Drawable;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageButton;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.Spinner;

public class EditTimer extends Activity {

	private EditText mTitleText, mMinuteText, mHourText;
	private Spinner toneSpinner;
	private MiniAppDb mMiniAppDb;
	
	
	public static final int ACTIVITY_ADD_MINI_APP = 1;

	Bundle info;
	long mRowId;

	/** Called when the activity is first created. */
	@Override
	public void onCreate(Bundle icicle) {
		super.onCreate(icicle);
		setContentView(R.layout.edittimer);
		
		mTitleText = (EditText) findViewById(R.id.title);

		Button addMiniAppButton = (Button) findViewById(R.id.addMiniAppButton);
		
		final Activity context = this;	
		addMiniAppButton.setOnClickListener(new View.OnClickListener() {
			/** Opens the EditTimer screen when an alarm is selected with the EDIT intent. **/
			public void onClick(View v) {
				Intent i = new Intent(context, AddMiniApp.class);

				startSubActivity(i, EditTimer.ACTIVITY_ADD_MINI_APP);
			}
        });		
				
		String requestedAction = getIntent().getAction() ;
		info = getIntent().getExtras(); //the bundle with all the alarm's info
		assert info != null;
		mRowId = info.getLong(AlarmsDbAdapter.KEY_ROWID);
		
		if (requestedAction.equals("EDIT")) {
			String title = info.getString(AlarmsDbAdapter.KEY_TITLE);
			//TODO need to get other info out of info bundle
			if (title != null) {
				mTitleText.setText(title);
				//TODO need to set other fields with info passed in
			}
		} else if (requestedAction.equals("CREATE")) {
			mTitleText.setText("New Timer");
			//TODO need to get a suitable default value for fields, esp. miniapps
		}
	
		
		/* Register confirm button to send data back to myAlarms screen */
		Button confirmButton = (Button) findViewById(R.id.editTimerConfirm);
		confirmButton.setOnClickListener(new View.OnClickListener() {

            public void onClick(View view) {
                Bundle bundle = new Bundle();
                int hours = new Integer(mHourText.getText().toString());
                int totalMinutes = hours * 60 +
                				new Integer(mMinuteText.getText().toString());
                Log.println(100, "PRINTED", "totalMinutes: " + totalMinutes);
                //Log.println(100, "PRINT", "EditTimer: mRowId="+mRowId);
                bundle.putLong(AlarmsDbAdapter.KEY_ROWID, mRowId);
                bundle.putString(AlarmsDbAdapter.KEY_TITLE, mTitleText.getText().toString());
                bundle.putInt(AlarmsDbAdapter.KEY_TIME, totalMinutes);
                bundle.putString(AlarmsDbAdapter.KEY_TONE, toneSpinner.getSelectedItem().toString());
                bundle.putInt(AlarmsDbAdapter.KEY_IS_TIMER, 1);
                //Log.println(100, "PRINT", "EditTimer: mTitleText="+mTitleText.getText().toString());
                
                setResult(RESULT_OK, null, bundle);
                finish();
            }
        });
		
		/* Set up TimeField */
		mHourText = (EditText) findViewById(R.id.hourField);
		mMinuteText = (EditText) findViewById(R.id.minuteField);

		int totalMinutes = info.getInt(AlarmsDbAdapter.KEY_TIME);

		if (totalMinutes!=0) {
			int hours = totalMinutes / 60;
			mHourText.setText(new Integer(hours).toString());
	        mMinuteText.setText(new Integer(totalMinutes%60).toString());
		} else {
			mHourText.setText("00");
	        mMinuteText.setText("00");
		}
		
		

		// set up Tone spinner
        toneSpinner = (Spinner) findViewById(R.id.spinner2);
        ArrayAdapter<CharSequence> adapter = ArrayAdapter.createFromResource(this, R.array.tone,
                android.R.layout.simple_spinner_item);
        adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        toneSpinner.setAdapter(adapter);
        String tone = info.getString(AlarmsDbAdapter.KEY_TONE);
        toneSpinner.setSelection(adapter.getPosition(tone));
        
        // create AddMiniApp button
		// miniapp alarmIds == Alarms' row Ids.  HACKY I KNOW :(((
		mMiniAppDb = new MiniAppDb(this, mRowId);
		mMiniAppDb.open();
		
		createMiniAppButton();
		createMiniAppList();
	}
	
	/**
	 * Setup everything necessary for the AddMiniApp button
	 */
	private void createMiniAppButton() {
		assert mRowId != -1;
		Button addMiniAppButton = (Button) findViewById(R.id.addMiniAppButton);
		
		final Activity context = this;
		
		addMiniAppButton.setOnClickListener(new View.OnClickListener() {
			
			public void onClick(View v) {
				Intent i = new Intent(context, AddMiniApp.class);
				i.putExtra(MiniAppDb.KEY_ALARM_ID, mRowId);
				startSubActivity(i, EditAlarm.ACTIVITY_ADD_MINI_APP);
			}
	    });	
	}
	
	/**
	 * Setup the list of Editable MiniApps.
	 */
	private void createMiniAppList() {
		
		
		Cursor c = mMiniAppDb.fetchMiniAppCursorByAlarmId();
		c.first();
		int count = c.count();
		LinearLayout layout = (LinearLayout) findViewById(R.id.miniAppLinearLayout);
		
		// clear existing children
		layout.removeAllViews();
		final Context ctx = this;
		
		for(c.first(); c.position() < count; c.next()) {
			String name = c.getString(c.getColumnIndex(MiniAppDb.KEY_MINIAPP_NAME));
			MiniAppActivity m = MiniAppActivity.miniAppFromString(name);
			int iconAddress = m.getIconAddress();
			
			LinearLayout.LayoutParams iconParams = 
                new LinearLayout.LayoutParams(40, 40);
            iconParams.setMargins(1,1,1,1);
            
            Drawable icon = this.getResources().getDrawable(m.getIconAddress());
			ImageButton iconButton = new ImageButton(this);
			iconButton.setPreferredHeight(40);
			iconButton.setPreferredWidth(40);
			iconButton.setImageDrawable(icon);
			iconButton.setClickable(true);
			iconButton.setAdjustViewBounds(true);
			
			// scale down images
			Matrix mat = new Matrix();
			mat.preScale(.6f, .6f);
			mat.preTranslate(-8f, -10f);
			iconButton.setImageMatrix(mat);
			iconButton.setScaleType(ImageView.ScaleType.MATRIX);
			// associate an instance of the miniApp with m
			iconButton.setTag(m);
	
			// fire up a new MiniApp screen if confirm button is hit.
			iconButton.setOnClickListener(
					new View.OnClickListener() {
						public void onClick(View v) {
	        		
							/** Opens the appropriate MiniApp Creation activity **/
							MiniAppActivity m = (MiniAppActivity) v.getTag();
							Class<MiniAppActivity> maClass = m.getActivityClass();
							Intent i = new Intent(ctx, maClass);
							i.putExtra(MiniAppDb.KEY_ALARM_ID, mRowId);
					
							mMiniAppDb.updateInstance(m.getNiceName());
							startSubActivity(i, AddMiniApp.ACTIVITY_ADD_MINIAPP);
						}
					}
			);
			layout.addView(iconButton, iconParams);
		}
		
	}
	
	
	protected void onActivityResult(int requestCode, int resultCode, String data, Bundle extras) {
		createMiniAppList();
	}
	
	
	public View.OnClickListener cancelButtonAction() {
		return new View.OnClickListener() {
			public void onClick(View view) {
                setResult(RESULT_CANCELED, null, null);
                finish();
            }
		};
	}
}