package edu.berkeley.cs160.alarmbuddy;

/* ===== MYALARMS.JAVA =====
 * 
 * Author: Daniel Gallagher
 * 
 * This activity represents the the "My Alarms" screen that users will see
 * first thing on starting the program from the Android main screen. It is
 * the gateway into all other activities that make up AlarmBuddy.
 * 
 * In the "My Alarms" screen users can edit a previously saved alarm
 * or request the creation of a new one. MyAlarms will call the EditAlarm
 * activity with the user's request. It also performs the actual
 * setting of the alarm, either when a user explicity turns a previously
 * saved alarm on in the "My Alarms" screen, or when the EditAlarm
 * activity returns a newly created alarm. * 
 * 
 * 1. Activity Lifecycle
 * 		a. onCreate
 * 		b. onResume
 * 		c. onFreeze
 * 		d. onPause
 * 2. Other Functions
 * 
 *  ===================== 
 */

//import com.google.android.samples.app.AlarmService;
//import com.google.android.samples.app.AlarmService_Alarm;

import java.util.ArrayList;

import android.app.Activity;
import android.app.AlarmManager;
import android.app.AlertDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.database.Cursor;
import android.graphics.Color;
import android.graphics.Matrix;
import android.graphics.drawable.Drawable;
import android.os.Bundle;
import android.os.SystemClock;
import android.util.Log;
import android.view.Menu;
import android.view.View;
import android.widget.AdapterView;
import android.widget.Button;
import android.widget.GridView;

public class MyAlarms extends Activity {

	public static final int ACTIVITY_CREATE=0;
	public static final int ACTIVITY_EDIT=1;

	private AlarmsDbAdapter mDbHelper;
	private Cursor mAlarmsCursor;
	GridView mGrid;
	private AlarmAdapter mAdapter;
	private ArrayList<Long> rowIDmap = new ArrayList<Long>(); //because adapter rowID and the real sqlite rowID are suckily different...

	/* 1. Activity Lifecycle */

	/** 1a. onCreate : Called when the activity is first created. */
	@Override
	public void onCreate(Bundle icicle) {
		super.onCreate(icicle);
		setContentView(R.layout.myalarms);

		mDbHelper = new AlarmsDbAdapter(this);
		mDbHelper.open();
		mGrid = (GridView) findViewById(R.id.myGrid);

		fillData();

		mGrid.setSelection(0);

		final Activity context = this;
		/* Register grid click to open up an alarm */
		mGrid.setOnItemClickListener(new AdapterView.OnItemClickListener() {

			/** Opens the EditAlarm screen when an alarm is selected with the EDIT intent. **/
			public void onItemClick(AdapterView parent, View v, int position, long id) {
				onAlarmClicked(parent, v, position, id);
			}
		});
		
		setupCreateButtons();
	}

	/** 1b. onResume : Called when the activity will start interacting 
	 * with the user. */
	@Override
	public void onResume() {
		super.onResume();
	}

	/** 1c. onFreeze : Allows you to save away your current state, 
	 * when your activity is being paused and another one resuming 
	 * to interact with the user. */
	@Override
	public void onFreeze(Bundle outState) {
	}

	/** 1d. onPause : Called when the system is about to start resuming 
	 * a previous activity.  */
	@Override
	public void onPause() {
		super.onPause();
	}

	private void fillData() {
		// Get all of the rows from the database and create the item list
		mAlarmsCursor = mDbHelper.fetchAllAlarms();
		startManagingCursor(mAlarmsCursor);

		//Create mapping array from adapter rowID to sqlite rowID
		rowIDmap.clear();
		mAlarmsCursor.first();
		for (int i=0; i<mAlarmsCursor.count(); i++) {
			long id = mAlarmsCursor.getLong(mAlarmsCursor.getColumnIndex(AlarmsDbAdapter.KEY_ROWID));
			String title = mAlarmsCursor.getString(mAlarmsCursor.getColumnIndex(AlarmsDbAdapter.KEY_TITLE));
			rowIDmap.add(i, id);
			Log.println(100, "print", "fillData(): "+title+"("+i+", "+id+") added to rowIDmap");
			mAlarmsCursor.next();
		}

		// Now create a simple cursor adapter and set it to display
		//mAdapter = new SimpleCursorAdapter(this, R.layout.myalarms_alarmbutton, mAlarmsCursor, from, to);
		mAdapter = new AlarmAdapter(this, mAlarmsCursor);
		int gridPosition = mGrid.getSelectedItemPosition();
		//setListAdapter(notes);
		mGrid.setAdapter(mAdapter);
		
		if(gridPosition != -1 && gridPosition < mAdapter.getCount()) {
			mGrid.setSelection(gridPosition);
		}
	}

	/** When EditAlarms is done, this method will be run to process the updated alarm. */
	@Override
	protected void onActivityResult(int requestCode, int resultCode, String data, Bundle extras) {
		super.onActivityResult(requestCode, resultCode, data, extras);

		if (resultCode == Activity.RESULT_OK) {
			long id = extras.getLong(AlarmsDbAdapter.KEY_ROWID);

			//Log.println(100, "PRINT", "MyAlarms: returned id="+id);

			String title = extras.getString(AlarmsDbAdapter.KEY_TITLE);

			//Log.println(100, "PRINT", "MyAlarms: returned title="+title);

			// active is handled separately from the fields in edit alarm.
			Cursor c = mDbHelper.fetchAlarm(id);
			int active = c.getInt(c.getColumnIndex(AlarmsDbAdapter.KEY_ACTIVE));
			
			int time = extras.getInt(AlarmsDbAdapter.KEY_TIME);

			String recurrence = extras.getString(AlarmsDbAdapter.KEY_RECURRENCE);
			int custom_recurrence = extras.getInt(AlarmsDbAdapter.KEY_CUSTOM_RECURRENCE);
			String tone = extras.getString(AlarmsDbAdapter.KEY_TONE);
			int isTimer = extras.getInt(AlarmsDbAdapter.KEY_IS_TIMER);
			if (requestCode == ACTIVITY_EDIT) {
				boolean x = mDbHelper.updateAlarm(id, title, time, active, recurrence, custom_recurrence, tone, isTimer);
				Log.println(100, "print", "after edit, updating db returns: "+x);
			} else if (requestCode == ACTIVITY_CREATE) {
				// TODO we create alarms immediately after hitting new alarm for now
				// we can fix this later
				// mDbHelper.createAlarm(title);
				mDbHelper.updateAlarm(id, title, time, active, recurrence, custom_recurrence, tone, isTimer);
			}
			//setAlarm(10);
		} else if (requestCode == ACTIVITY_CREATE && 
				resultCode == Activity.RESULT_CANCELED) {
			// do stuff
		}
		fillData();
	}

	private void setAlarm (int timeTillGoOffSeconds) {
		Intent intent = new Intent(MyAlarms.this, AlarmSetOff.class);

		int selection = mGrid.getSelectedItemPosition();
		long alarmId = rowIDmap.get(selection);
		intent.putExtra(MiniAppDb.KEY_ALARM_ID, alarmId);
		// We want the alarm to go off 10 seconds from now.
		long firstTime = SystemClock.elapsedRealtime() + timeTillGoOffSeconds*1000;

		// Schedule the alarm!
		AlarmManager am = (AlarmManager)getSystemService(ALARM_SERVICE);
		am.set(AlarmManager.ELAPSED_REALTIME_WAKEUP,
				firstTime, intent);
	}

	// Called only the first time the options menu is displayed.
	// Create the menu entries.
	// Menu adds items in the order shown.
	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		super.onCreateOptionsMenu(menu);

		// Parameters for menu.add are:
		// group -- Not used here.
		// id -- Used only when you want to handle and identify the click yourself.
		// title
		menu.add(0, 0, "New Alarm");
		menu.add(1, 5, "New Timer");
		menu.add(2, 1, "Edit");
		menu.add(3, 2, "Enable/Disable");
		menu.add(4, 3, "Delete");
		menu.add(5, 4, "Delete All");
		return true;
	}

	// Activity callback that lets your handle the selection in the class.
	// Return true to indicate that you've got it, false to indicate
	// that it should be handled by a declared handler object for that
	// item (handler objects are discouraged for reasons of efficiency).
	@Override
	public boolean onOptionsItemSelected(Menu.Item item){
		int sel = mGrid.getSelectedItemPosition();

		switch (item.getId()) {
		case 0: //NEW
			startEditAlarm(sel, ACTIVITY_CREATE);
			return true;
		case 1: //EDIT
			startEditAlarm(sel, ACTIVITY_EDIT);
			return true;
		case 2: //ENABLE/DISABLE
			setAlarm(1); //this should be changed to accept the ID of the alarm selected
			long gridPosition = mGrid.getSelectedItemPosition();
			long rowId = mAdapter.getItemId((int)gridPosition);
			Cursor c = mDbHelper.fetchAlarm(rowId);
			boolean active = 1 == c.getInt(c.getColumnIndex(AlarmsDbAdapter.KEY_ACTIVE)); 
			c.updateInt(c.getColumnIndex(AlarmsDbAdapter.KEY_ACTIVE), 
					(active) ? 0 : 1);
			c.commitUpdates();
			fillData();
			return true;
		case 3: //DELETE
			deleteAlarm(sel);
			fillData();
			return true;
		case 4: //DELETE ALL
			for (int i=0; i<mGrid.getCount(); i++) {
				deleteAlarm(i);
			}
			fillData();
			return true;
		case 5: //NEW TIMER
			startEditTimer(sel, ACTIVITY_CREATE);
			return true;
		}
		return false;
	}

	private void startEditAlarm (int selection, int requestCode) {
		Intent i = new Intent(this, EditAlarm.class);
		long id;
		if (requestCode==ACTIVITY_CREATE) {
			i.setAction("CREATE");			
			id = mDbHelper.createAlarm("New Alarm");
		} else {
			i.setAction("EDIT");
			id = rowIDmap.get(selection);
		}

		//Log.println(100, "print", "startEditAlarms: id found to be: "+id);
		Cursor alarm = mDbHelper.fetchAlarm(id);

		i.putExtra(AlarmsDbAdapter.KEY_ROWID, id);
		i.putExtra(AlarmsDbAdapter.KEY_TITLE, alarm.getString(alarm.getColumnIndex(AlarmsDbAdapter.KEY_TITLE)));
		i.putExtra(AlarmsDbAdapter.KEY_TIME, alarm.getInt(alarm.getColumnIndex(AlarmsDbAdapter.KEY_TIME)));
		i.putExtra(AlarmsDbAdapter.KEY_ACTIVE, alarm.getInt(alarm.getColumnIndex(AlarmsDbAdapter.KEY_ACTIVE)));
		i.putExtra(AlarmsDbAdapter.KEY_RECURRENCE, alarm.getString(alarm.getColumnIndex(AlarmsDbAdapter.KEY_RECURRENCE)));
		i.putExtra(AlarmsDbAdapter.KEY_CUSTOM_RECURRENCE, alarm.getInt(alarm.getColumnIndex(AlarmsDbAdapter.KEY_CUSTOM_RECURRENCE)));
		i.putExtra(AlarmsDbAdapter.KEY_TONE, alarm.getString(alarm.getColumnIndex(AlarmsDbAdapter.KEY_TONE)));
		i.putExtra(AlarmsDbAdapter.KEY_IS_TIMER, alarm.getInt(alarm.getColumnIndex(AlarmsDbAdapter.KEY_IS_TIMER)));

		startSubActivity(i, requestCode);
	}

	private void startEditTimer (int selection, int requestCode) {
		Intent i = new Intent(this, EditTimer.class);
		long id;
		if (requestCode==ACTIVITY_CREATE) {
			i.setAction("CREATE");			
			id = mDbHelper.createTimer("New Timer"); //change this to some new createTimer
		} else {
			i.setAction("EDIT");
			id = rowIDmap.get(selection);
		}

		//Log.println(100, "print", "startEditAlarms: id found to be: "+id);
		Cursor alarm = mDbHelper.fetchAlarm(id);

		// Pass in all the existing values to the Edit Timer Activity
		// we pass this in regardless of editing an existing alarm or creating
		// a new one.
		i.putExtra(AlarmsDbAdapter.KEY_ROWID, id);
		i.putExtra(AlarmsDbAdapter.KEY_TITLE, alarm.getString(alarm.getColumnIndex(AlarmsDbAdapter.KEY_TITLE)));
		i.putExtra(AlarmsDbAdapter.KEY_TIME, alarm.getInt(alarm.getColumnIndex(AlarmsDbAdapter.KEY_TIME)));
		i.putExtra(AlarmsDbAdapter.KEY_ACTIVE, alarm.getInt(alarm.getColumnIndex(AlarmsDbAdapter.KEY_ACTIVE)));
		i.putExtra(AlarmsDbAdapter.KEY_TONE, alarm.getString(alarm.getColumnIndex(AlarmsDbAdapter.KEY_TONE)));
		i.putExtra(AlarmsDbAdapter.KEY_IS_TIMER, alarm.getInt(alarm.getColumnIndex(AlarmsDbAdapter.KEY_IS_TIMER)));

		startSubActivity(i, requestCode);
	}
	
	private void disableAlarm (int selection) {
		Cursor c = mDbHelper.fetchAlarm(rowIDmap.get(selection));
		c.updateInt(c.getColumnIndex(AlarmsDbAdapter.KEY_ACTIVE), 0);
		c.commitUpdates();
		fillData();
	}
	
	private void enableAlarm (int selection) {
		Cursor c = mDbHelper.fetchAlarm(rowIDmap.get(selection));
		c.updateInt(c.getColumnIndex(AlarmsDbAdapter.KEY_ACTIVE), 1);
		c.commitUpdates();
		fillData();
	}

	private void deleteAlarm (int selection) {
		long id = mAdapter.getItemId(selection);
		mDbHelper.deleteAlarm(id);
	}

	private void onAlarmClicked(AdapterView parent, View v, final int position, long id) {
		final Cursor alarm = mDbHelper.fetchAlarm(rowIDmap.get(position));
		String name = alarm.getString(alarm.getColumnIndex(AlarmsDbAdapter.KEY_TITLE));
		int enabled = alarm.getInt(alarm.getColumnIndex(AlarmsDbAdapter.KEY_ACTIVE));
		// removed alarm name from the Msgs because long names would break
		// prevent you from moving left and right between buttons.  Yay Android.
		String EnableMsg = "Enable";
		String DisableMsg = "Disable";
		String EditMsg = "Edit";
		final Activity context = this;
		
		//Create the popup dialogue asking whether user wants to edit or enable/disable clicked alarm
		AlertDialog.Builder b = new AlertDialog.Builder(this)
		.setTitle("Alarm: '"+name+"'")
		.setNegativeButton(EditMsg, new DialogInterface.OnClickListener() {
			public void onClick(DialogInterface dialog, int whichButton) {
				Intent i;
				boolean isTimerBool = 1 == alarm.getInt(alarm.getColumnIndex(AlarmsDbAdapter.KEY_IS_TIMER));
				if(isTimerBool) {
					i = new Intent(context, EditTimer.class);
				} else {
					i = new Intent(context, EditAlarm.class);
				}
				i.setAction("EDIT");
				i.putExtra(AlarmsDbAdapter.KEY_ROWID, rowIDmap.get(position));
				i.putExtra(AlarmsDbAdapter.KEY_TITLE, alarm.getString(alarm.getColumnIndex(AlarmsDbAdapter.KEY_TITLE)));
				i.putExtra(AlarmsDbAdapter.KEY_TIME, alarm.getInt(alarm.getColumnIndex(AlarmsDbAdapter.KEY_TIME)));
				i.putExtra(AlarmsDbAdapter.KEY_ACTIVE, alarm.getInt(alarm.getColumnIndex(AlarmsDbAdapter.KEY_ACTIVE)));
				i.putExtra(AlarmsDbAdapter.KEY_RECURRENCE, alarm.getString(alarm.getColumnIndex(AlarmsDbAdapter.KEY_RECURRENCE)));
				i.putExtra(AlarmsDbAdapter.KEY_CUSTOM_RECURRENCE, alarm.getInt(alarm.getColumnIndex(AlarmsDbAdapter.KEY_CUSTOM_RECURRENCE)));
				i.putExtra(AlarmsDbAdapter.KEY_TONE, alarm.getString(alarm.getColumnIndex(AlarmsDbAdapter.KEY_TONE)));
				i.putExtra(AlarmsDbAdapter.KEY_IS_TIMER, alarm.getInt(alarm.getColumnIndex(AlarmsDbAdapter.KEY_IS_TIMER)));
				startSubActivity(i, ACTIVITY_EDIT);
			}
		});
		if (enabled == 1) {
			b.setPositiveButton(DisableMsg, new DialogInterface.OnClickListener() {
				public void onClick(DialogInterface dialog, int whichButton) {
					disableAlarm(position);
				}
			});
		} else {
			b.setPositiveButton(EnableMsg, new DialogInterface.OnClickListener() {
				public void onClick(DialogInterface dialog, int whichButton) {
					enableAlarm(position);
				}
			});
		}
		b.show();
	}

    private void setupCreateButtons() {
		Button alarmButton = (Button) findViewById(R.id.createAlarmButton);
        Button timerButton = (Button) findViewById(R.id.createTimerButton);
        
        Matrix mat = new Matrix();
		mat.preScale(.6f, .6f);
		mat.preTranslate(-8f, -10f);
		Drawable bgAlarm = this.getResources().getDrawable(R.drawable.bg_blue_active);
		Drawable bgTimer = this.getResources().getDrawable(R.drawable.bg_green_active);
		
        
        alarmButton.setOnClickListener(new View.OnClickListener() {
        	public void onClick(View v) {
        		// nothing selected, can pass 0 for selection
        		startEditAlarm(0, ACTIVITY_CREATE);
            }
        });
        
        timerButton.setOnClickListener(new View.OnClickListener() {
        	public void onClick(View v) {
        		// nothing selected, can pass 0 for selection
        		startEditTimer(0, ACTIVITY_CREATE);
        		
            }
        });
	}
	
}