package edu.berkeley.cs160.alarmbuddy;
/***
 * SingleWeatherInfoView.java class from tutorial at anddev.org.
 * Edited by Andry Jong
 */
import java.io.BufferedInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.net.URLConnection;
import java.util.Map;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.util.AttributeSet;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;

/**
 * The View capable of showing a WeatehrIcon + a Temperature-TextView.
 */
public class SingleWeatherInfoView extends LinearLayout {

	// ===========================================================
	// Fields
	// ===========================================================
	private TextView myWeatherDay = null;
	private ImageView myWeatherImageView = null;
	private TextView myTempTextView = null;

	// ===========================================================
	// Constructors
	// ===========================================================

	public SingleWeatherInfoView(Context context) {
		super(context);
	}

	public SingleWeatherInfoView(Context context, AttributeSet attrs,Map inflateParams) {
		super(context, attrs, inflateParams);
		this.myWeatherDay = new TextView(context);
		this.myWeatherDay.setText("?");
		/* Setup the ImageView that will show weather-icon. */
		this.myWeatherImageView = new ImageView(context);
		this.myWeatherImageView.setImageDrawable(getResources().getDrawable(
				R.drawable.dunno));

		/* Setup the textView that will show the temperature. */
		this.myTempTextView = new TextView(context);
		this.myTempTextView.setText("? F");
		//this.myTempTextView.setTextSize(16);
		//this.myTempTextView.setTypeface(Typeface
		//		.create("Tahoma", Typeface.BOLD));

		/* Add child views to this object. */
		this.addView(this.myWeatherDay, new LinearLayout.LayoutParams(LayoutParams.WRAP_CONTENT, LayoutParams.WRAP_CONTENT));
		this.addView(this.myWeatherImageView, new LinearLayout.LayoutParams(LayoutParams.WRAP_CONTENT, LayoutParams.WRAP_CONTENT));
		this.addView(this.myTempTextView, new LinearLayout.LayoutParams(LayoutParams.WRAP_CONTENT, LayoutParams.WRAP_CONTENT));
	}

	// ===========================================================
	// Getter & Setter
	// ===========================================================

	public void reset() {
		this.myWeatherDay.setText("?");
		this.myWeatherImageView.setImageDrawable(getResources().getDrawable(R.drawable.dunno));
		this.myTempTextView.setText("? F");
	}

	/** Sets the Child-ImageView of this to the URL passed. */
	public void setRemoteImage(URL aURL) {
		try {
			URLConnection conn = aURL.openConnection();
			conn.connect();
			InputStream is = conn.getInputStream();
			BufferedInputStream bis = new BufferedInputStream(is);
			Bitmap bm = BitmapFactory.decodeStream(bis);
			bis.close();
			is.close();
			this.myWeatherImageView.setImageBitmap(bm);
		} catch (IOException e) {
			/* Reset to 'Dunno' on any error. */
			this.myWeatherImageView.setImageDrawable(getResources()
					.getDrawable(R.drawable.dunno));
		}
	}

	public void setTempCelcius(int aTemp) {
		this.myTempTextView.setText("" + aTemp + " C");
	}

	public void setTempFahrenheit(int aTemp) {
		this.myTempTextView.setText("" + aTemp + " F");
	}

	public void setTempFahrenheitMinMax(int aMinTemp, int aMaxTemp) {
		this.myTempTextView.setText("" + aMaxTemp + "|" + aMinTemp + " F");
	}

	public void setTempCelciusMinMax(int aMinTemp, int aMaxTemp) {
		this.myTempTextView.setText("" + aMaxTemp + "|" + aMinTemp + " C");
	}

	public void setTempString(String aTempString) {
		this.myTempTextView.setText(aTempString);
	}
	
	public void setDay(String string) {
		this.myWeatherDay.setText(string);
	}
}
