package edu.berkeley.cs160.alarmbuddy;
/**
 * 	To Do list Manager... from the Notepad example.
 * 	Very little change was made to the layout.
 *  -- andry jong
 */
import android.app.ListActivity;
import android.content.Intent;
import android.database.Cursor;
import android.os.Bundle;
import android.view.Menu;
import android.view.View;
import android.view.Menu.Item;
import android.widget.Button;
import android.widget.ListView;
import android.widget.SimpleCursorAdapter;


public class ToDo extends ListActivity {
    private static final int ACTIVITY_CREATE=0;
    private static final int ACTIVITY_EDIT=1;
    private static final int DELETE_ID = Menu.FIRST;
    
    private ToDoDbAdapter mDbHelper;
    private long mAlarmId = -1;
    /** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle icicle) {
        super.onCreate(icicle);
        setContentView(R.layout.note_list);
        
        // figure out what alarm is currently using the miniApp so we
        // can retrieve the appropriate data
    	Bundle extras = getIntent().getExtras();    	
    	mAlarmId = extras.getLong(MiniAppDb.KEY_ALARM_ID);
        
        mDbHelper = new ToDoDbAdapter(this, mAlarmId);
        mDbHelper.open();
        fillData();
        
        Button ADD = (Button) findViewById(R.id.addToDo);
        
        ADD.setOnClickListener(new View.OnClickListener() {
        	public void onClick(View v) {
        		createNote();
            }
        });
        
        setupCancelConfirmButtons();
        
    }
    
    private void fillData() {
        // Get all of the rows from the database and create the item list
        Cursor notesCursor = mDbHelper.fetchAllNotes();
        startManagingCursor(notesCursor);
        
        // Create an array to specify the fields we want to display in the list (only TITLE)
        String[] from = new String[]{ToDoDbAdapter.KEY_TITLE};
        
        // and an array of the fields we want to bind those fields to (in this case just text1)
        int[] to = new int[]{R.id.text1};
        
        // Now create a simple cursor adapter and set it to display
        SimpleCursorAdapter notes = 
        	    new SimpleCursorAdapter(this, R.layout.notes_row, notesCursor, from, to);
        setListAdapter(notes);
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        super.onCreateOptionsMenu(menu);
        menu.add(0, DELETE_ID, R.string.menu_delete);
        return true;
    }
    
    @Override
    public boolean onMenuItemSelected(int featureId, Item item) {
        switch(item.getId()) {
        case DELETE_ID:
            mDbHelper.deleteNote(getListView().getSelectedItemId());
            fillData();
            return true;
        }
       
        return super.onMenuItemSelected(featureId, item);
    }
    
    private void createNote() {
        Intent i = new Intent(this, ToDoEdit.class);
        i.putExtra(MiniAppDb.KEY_ALARM_ID, mAlarmId);
        i.putExtra(ToDoEdit.KEY_NEW_NOTE, true);
        startSubActivity(i, ACTIVITY_CREATE);
    }
    
    @Override
    protected void onListItemClick(ListView l, View v, int position, long id) {
        super.onListItemClick(l, v, position, id);
        Intent i = new Intent(this, ToDoEdit.class);
        i.putExtra(ToDoDbAdapter.KEY_ROWID, id);
        i.putExtra(MiniAppDb.KEY_ALARM_ID, mAlarmId);
        i.putExtra(ToDoEdit.KEY_NEW_NOTE, false);
        startSubActivity(i, ACTIVITY_EDIT);
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, String data, Bundle extras) {
        super.onActivityResult(requestCode, resultCode, data, extras);
        fillData();
    }
    
    private void setupCancelConfirmButtons() {
		Button cancelButton = (Button) findViewById(R.id.cancel);
        Button confirmButton = (Button) findViewById(R.id.confirm);
        
        confirmButton.setOnClickListener(new View.OnClickListener() {
        	public void onClick(View v) {
        		setResult(RESULT_OK, null, null);
                finish();
            }
        });
        
        cancelButton.setOnClickListener(new View.OnClickListener() {
        	public void onClick(View v) {
        		setResult(RESULT_CANCELED, null, null);
                finish();
            }
        });
	}
    
}
