package edu.berkeley.cs160.alarmbuddy;

/* ===== ViewMiniApps.JAVA =====
 *  Author: Jessica Fitzgerald
 *  The screen that allows you to browse all the mini-apps once the
 *  alarm has gone off
 *  
 *  ===================== 
 */


import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.database.Cursor;
import android.graphics.Color;
import android.graphics.drawable.Drawable;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.view.Window;
import android.widget.AdapterView;
import android.widget.GridView;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;


public class ViewMiniApps extends Activity {

	private MiniAppDb mMiniAppDb;
	private AlarmsDbAdapter mDb;
	private long mAlarmId = -1;
	private MiniAppActivity[] mMiniAppInstances;
	
	ImageView weatherPreview;
	GridView mGrid;
	Cursor mMiniAppsCursor;
	
	
	@Override
	protected void onCreate(Bundle icicle)
	    {
		super.onCreate(icicle);

		requestWindowFeature(Window.FEATURE_NO_TITLE);
		
	    setContentView(R.layout.view_mini_app);
		
	    
	    
	    Bundle extras = getIntent().getExtras();
		assert extras != null;
		mAlarmId = extras.getLong(MiniAppDb.KEY_ALARM_ID);
		
		mDb = new AlarmsDbAdapter(this);
		mDb.open();
		mMiniAppDb = new MiniAppDb(this, mAlarmId);		
		mMiniAppDb.open();
	    
		Cursor c = mDb.fetchAlarm(mAlarmId); 
		String alarmName = c.getString(c.getColumnIndex(AlarmsDbAdapter.KEY_TITLE));
		
		TextView nameControl = (TextView) findViewById(R.id.alarmNameControl);
		nameControl.setText(alarmName);
		

		
		Log.println(100, "wtf", "wtf: viewMiniApps: alarmId=" + mAlarmId);
		setupGrid();
		
		//createMiniAppList();
		
	    //weatherPreview = (ImageView) findViewById(R.id.weatherPreview2);
	    //weatherPreview.setImageResource(R.drawable.yahoo_weather);
		//weatherPreview.setVisibility(View.VISIBLE);
		
	    }

	
	private void setupGrid() {
		
		mGrid = (GridView) findViewById(R.id.viewMiniAppsGrid);
		
		final Context ctx = this;
		
		// fire up a new MiniApp screen if confirm button is hit.
		mGrid.setOnItemClickListener(new AdapterView.OnItemClickListener() {
			/** Opens the appropriate MiniApp Creation activity **/
			public void onItemClick(AdapterView parent, View v, int position, long id) {
				MiniAppActivity ma = mMiniAppInstances[(int)id];
				Class<MiniAppActivity> maClass = ma.getActivityClass();
				Intent i = new Intent(ctx, maClass);
				i.putExtra(MiniAppDb.KEY_ALARM_ID, mAlarmId);
				i.setAction("VIEW");
				
				// create the entry in the database
				// this should be deleted in the event of a cancel.
				mMiniAppDb.updateInstance(ma.getNiceName());
				startSubActivity(i, 0);
			}
        });
		
		fillData();
	}
	
	
	/**
	 * Read the available MiniApps from the DataBase.
	 */
	private void fillData() {
				
		// Get all of the rows from the database and create the item list
		Cursor c = mMiniAppDb.fetchMiniAppCursorByAlarmId();
		startManagingCursor(c);
		int count = c.count();
		
		mMiniAppInstances = new MiniAppActivity[count];
		int i = 0;
		for(c.first(); c.position() < count; c.next(),i++) {
			String miniAppName = c.getString(c.getColumnIndex(MiniAppDb.KEY_MINIAPP_NAME));
			
			MiniAppActivity m = MiniAppActivity.miniAppFromString(miniAppName);
			mMiniAppInstances[i] = m;
			String icon = m.getIconPath();
		}
		c.first();		
		MiniAppAdapter maAdapter = new MiniAppAdapter(this, mMiniAppInstances);		
		mGrid.setAdapter(maAdapter);

	}
	
	
	/**
	 * Setup the list of Editable MiniApps.
	 */
	private void createMiniAppList() {
		
		
		Cursor c = mMiniAppDb.fetchMiniAppCursorByAlarmId();
		c.first();
		int count = c.count();
		LinearLayout layout = (LinearLayout) findViewById(R.id.viewMiniAppsLinearLayout);
		
		// clear existing children
		layout.removeAllViews();
		final Context ctx = this;
		
		for(c.first(); c.position() < count; c.next()) {
			
			// setup variables 
			String miniAppName = c.getString(c.getColumnIndex(MiniAppDb.KEY_MINIAPP_NAME));
			Log.println(100, "wtf", "wtf: ViewMiniApps: miniAppName: " + miniAppName);
			MiniAppActivity m = MiniAppActivity.miniAppFromString(miniAppName);
			int iconAddress = m.getIconAddress();
			
			// create group containing images + text
			LinearLayout miniAppGroup = new LinearLayout(this);
			miniAppGroup.setOrientation(LinearLayout.VERTICAL);
			LinearLayout.LayoutParams miniAppGroupParams = 
	            new LinearLayout.LayoutParams(80, 80);
	        miniAppGroupParams.setMargins(1,1,1,1);
	
	        // create text control
	        TextView nameControl = new TextView(this);
	        nameControl.setText(miniAppName);
	        nameControl.setTextSize( 14f );
            nameControl.setTextColor(Color.WHITE);
	
			LinearLayout.LayoutParams nameControlParams = 
	            new LinearLayout.LayoutParams(LinearLayout.LayoutParams.WRAP_CONTENT,
	            		LinearLayout.LayoutParams.FILL_PARENT);
			nameControlParams.setMargins(1,1,1,1);
            
	        
	        
	        // create icon control
			LinearLayout.LayoutParams iconParams = 
	            new LinearLayout.LayoutParams(50, 50);
	        iconParams.setMargins(1,1,1,1);
	        
	        Drawable icon = this.getResources().getDrawable(m.getIconAddress());
			ImageView iconButton = new ImageView(this);
			iconButton.setPreferredHeight(50);
			iconButton.setPreferredWidth(50);
			iconButton.setBackground(icon);
			iconButton.setClickable(true);
			//iconButton.setAdjustViewBounds(true);
			// scale down images
			// Matrix mat = new Matrix();
			// mat.preScale(.6f, .6f);
			// mat.preTranslate(-8f, -10f);
			// iconButton.setImageMatrix(mat);
			// iconButton.setScaleType(ImageView.ScaleType.MATRIX);
			// associate an instance of the miniApp with m
			iconButton.setTag(m);
	
			// fire up a new MiniApp screen if confirm button is hit.
			miniAppGroup.setOnClickListener(
					new View.OnClickListener() {
						public void onClick(View v) {
	        		
							/** Opens the appropriate MiniApp Creation activity **/
							MiniAppActivity m = (MiniAppActivity) v.getTag();
							Class<MiniAppActivity> maClass = m.getActivityClass();
							Intent i = new Intent(ctx, maClass);
							i.putExtra(MiniAppDb.KEY_ALARM_ID, mAlarmId);
					
							mMiniAppDb.updateInstance(m.getNiceName());
							startSubActivity(i, AddMiniApp.ACTIVITY_ADD_MINIAPP);
						}
					}
			);
			miniAppGroup.addView(nameControl, nameControlParams);
			miniAppGroup.addView(iconButton, iconParams);
			layout.addView(miniAppGroup, miniAppGroupParams);
		}
		
	}
		
	
}
	
