package edu.berkeley.cs160.alarmbuddy;

/**	This is the Weather Application page.
 * 	Gets zip code from user, generate weather forecast summary.
 * 	
 * 	Right now it always returns the same summary (from yahoo weather).
 * 	Still doesn't check zip code's validity. 
 * 	--- andry jong ---
 **/

import android.content.Context;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageView;

public class Weather extends MiniAppActivity {
	
	public static final String NICE_NAME = "Weather";
	public static final String KEY_ZIPCODE = "zipcode";
	
	// key for the rowId of this alarm in weather, if we're editing
	// an existing MiniApp Instance.
	public static final String KEY_INSTANCE_ROWID = "_id";
	
	
	// UI Elements
	private EditText mZipCode;
	
	// the image view for the hacky weather preview image
	private ImageView weatherPreview;
	
	// id of the alarm using this miniApp
	private long mAlarmId = -1;
	
	// database holding all instances of the Weather MiniApp
	protected static WeatherDb mDb;
	
    /** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle icicle) {
        super.onCreate(icicle);
        setContentView(R.layout.weather);

        mDb = new WeatherDb(this);
        mDb.open();

        mZipCode = (EditText) findViewById(R.id.zipEntry);

        // figure out what alarm is currently using the miniApp so we
        // can retrieve the appropriate data
    	Bundle extras = getIntent().getExtras();    	
    	mAlarmId = extras.getLong(MiniAppDb.KEY_ALARM_ID);
    	String zipCodeValue = mDb.getInstanceZipcode(mAlarmId);
    	mZipCode.setText(zipCodeValue);
        
        weatherPreview = (ImageView) findViewById(R.id.weatherPreview);
        Button SEARCH = (Button) findViewById(R.id.zipOK);
        Button mCancelButton = (Button) findViewById(R.id.zipCancel);
        Button mConfirmButton = (Button) findViewById(R.id.zipConfirm);
        
        // unpack the icicle's zipcode if an entry exists
        resetZipCodeField(icicle);
                
        mConfirmButton.setOnClickListener(new View.OnClickListener() {
        	public void onClick(View v) {
        		onConfirmButtonClick();
            }
        });
        
        mCancelButton.setOnClickListener(new View.OnClickListener() {
        	public void onClick(View v) {
        		onCancelButtonClick();
            }
        });
        
        SEARCH.setOnClickListener(new View.OnClickListener() {
        	public void onClick(View v) {
        		onTestButtonClick();
            }
        });
    }
    
    public void onTestButtonClick() {
    	weatherPreview.setImageResource(R.drawable.yahoo_weather_preview);
    	weatherPreview.setVisibility(View.VISIBLE);
    }
    
    /**
     * Returns to the addMiniApp after setting this up is done.
     */
    public void onConfirmButtonClick() {
    	assert mAlarmId != -1;
    	mDb.updateInstance(mAlarmId, mZipCode.getText().toString());
    	Bundle result = new Bundle();
    	// result.putLong(MiniAppDb.KEY_INSTANCE_ID, newRowId);
    	setResult(RESULT_OK, null, null);
        finish();
    }
    
    /**
     * Returns to the addMiniApp after setting this up is done.
     */
    public void onCancelButtonClick() {
    	setResult(RESULT_CANCELED, null, null);
        finish();
    }
    
    /**
     * Set the text of the zipcode to whatever it last was
     * @param b bundle that may have a zipcode entry
     */
    public void resetZipCodeField(Bundle b) {
    	if(b == null) return;
    	if(b.containsKey(KEY_ZIPCODE)){
        	mZipCode.setText(b.getString(KEY_ZIPCODE)); 
        } else {
        	if(b.containsKey(KEY_INSTANCE_ROWID)) {
        		mZipCode.setText(mDb.getZipCode(b.getLong(KEY_INSTANCE_ROWID)));	
        	}
        }
    }
    
    @Override
    protected void onPause() {
        super.onPause();
    }
 
    @Override
    protected void onResume() {
        super.onResume();
    }
    
	@Override
	//pack up whatever's entered in the zipcode field
	protected void onFreeze(Bundle outState) {
		// TODO Auto-generated method stub
		super.onFreeze(outState);
		outState.putString(KEY_ZIPCODE,
				((EditText) findViewById(R.id.zipEntry)).getText().toString());
		
	}
	
	public String getIconPath() {
    	return "res/drawable/icon_weather";
    }
	
	public int getIconAddress() {
    	return R.drawable.icon_weather;
    }
    
	public String getNiceName() {
    	return NICE_NAME;
    }
    
    public void startEditActivity(Context ctx, int rowId) {
    	
    }
    
    public void startAddActivity(Context ct) {
    	
    }
    
    public void startAlarmOffActivity(Context ct, int rowId) {
    
    }
    
    public Weather getInstance(long instanceId) {
    	return null;
    }
    
    public void deleteInstance(long alarmId) {
    	mDb.deleteInstance(alarmId);
    }
    
    
}