package edu.berkeley.cs160.alarmbuddy;

/***
 * New Working Weather application.
 * Credit to tutorial at anddev.org for the help.
 * Created and editted by Andry Jong
 */
import java.net.MalformedURLException;
import java.net.URL;

import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.InputSource;
import org.xml.sax.XMLReader;

import android.content.SharedPreferences;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;

public class WeatherReal extends MiniAppActivity {
	
	EditText zipCode;
	String zip;
	public static final String NICE_NAME = "Weather 2.0"; 
    /** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle icicle) {
        super.onCreate(icicle);
        setContentView(R.layout.weather_real);
        
        Button SEARCH = (Button) findViewById(R.id.zipOK);
                
        SEARCH.setOnClickListener(new OnClickListener() {
        	public void onClick(View v) {
        		onSearchRequest();
            }
        });
    }
    
    public void onSearchRequest() {	
    	URL url;
    	try {
    		/* Get what user typed to the EditText. */
    		zipCode = (EditText) findViewById(R.id.zip_Entry);
            zip = zipCode.getText().toString();
    		String searchQuery = "http://www.google.com/ig/api?weather=" + zip;
    		url = new URL(searchQuery.replace(" ", "%20"));

    		/* Get a SAXParser from the SAXPArserFactory. */
    		SAXParserFactory spf = SAXParserFactory.newInstance();
    		SAXParser sp = spf.newSAXParser();
    		XMLReader xr = sp.getXMLReader();

    		GoogleWeatherHandler gwh = new GoogleWeatherHandler();
    		xr.setContentHandler(gwh);

    		/* Parse the xml-data our URL-call returned. */
    		xr.parse(new InputSource(url.openStream()));

    		/* Our Handler now provides the parsed weather-data to us. */
    		WeatherSet ws = gwh.getWeatherSet();

    		/* Update the SingleWeatherInfoView with the parsed data. */

    		updateWeatherInfoView(R.id.weather_1, ws.getWeatherForecastConditions().get(0));
    		updateWeatherInfoView(R.id.weather_2, ws.getWeatherForecastConditions().get(1));
    		updateWeatherInfoView(R.id.weather_3, ws.getWeatherForecastConditions().get(2));
    		updateWeatherInfoView(R.id.weather_4, ws.getWeatherForecastConditions().get(3));

    	} catch (Exception e) {
    		resetWeatherInfoViews();
    		Log.e("WeatherForcaster", "WeatherQueryError", e);
    	}
    }
    
    private void updateWeatherInfoView(int aResourceID,WeatherForecastCondition aWFIS) 
    	throws MalformedURLException {
		
		/* Construct the Image-URL. */
		URL imgURL = new URL("http://www.google.com" + aWFIS.getIconURL());
		((SingleWeatherInfoView) findViewById(aResourceID)).setRemoteImage(imgURL);

		int tempMin = aWFIS.getTempMinCelsius();
		int tempMax = aWFIS.getTempMaxCelsius();

		tempMin = celsiusToFahrenheit(tempMin);
		tempMax = celsiusToFahrenheit(tempMax);
		((SingleWeatherInfoView) findViewById(aResourceID)).setTempFahrenheitMinMax(tempMin, tempMax);
		
		String day = aWFIS.getDayofWeek();
		((SingleWeatherInfoView) findViewById(aResourceID)).setDay(day);
	}

	private void updateWeatherInfoView(int aResourceID,WeatherCurrentCondition aWCIS) 
		throws MalformedURLException {
		
		/* Construct the Image-URL. */
		URL imgURL = new URL("http://www.google.com" + aWCIS.getIconURL());
		((SingleWeatherInfoView) findViewById(aResourceID)).setRemoteImage(imgURL);
		((SingleWeatherInfoView) findViewById(aResourceID)).setTempFahrenheit(aWCIS.getTempFahrenheit());
		((SingleWeatherInfoView) findViewById(aResourceID)).setDay(aWCIS.getDayofWeek());
	}

	private void resetWeatherInfoViews() {
		((SingleWeatherInfoView)findViewById(R.id.weather_1)).reset();
		((SingleWeatherInfoView)findViewById(R.id.weather_2)).reset();
		((SingleWeatherInfoView)findViewById(R.id.weather_3)).reset();
		((SingleWeatherInfoView)findViewById(R.id.weather_4)).reset();
	}
	
	public int celsiusToFahrenheit(int tCelsius) {
		return (int) ((9.0f / 5.0f) * tCelsius + 32);
	}
	
	/**
     * Upon being resumed we can retrieve the current state.  This allows us
     * to update the state if it was changed at any time while paused.
     */
    @Override
    protected void onResume() {
        super.onResume();

        SharedPreferences prefs = getPreferences(0); 
        String restoredText = prefs.getString("search_text", null);
        if (restoredText != null) {
        	EditText searchText = (EditText) findViewById(R.id.zip_Entry);
            searchText.setText(restoredText, TextView.BufferType.EDITABLE);

            int selectionStart = prefs.getInt("selection-start", -1);
            int selectionEnd = prefs.getInt("selection-end", -1);
            if (selectionStart != -1 && selectionEnd != -1) {
                searchText.setSelection(selectionStart, selectionEnd);
            }
        }
    }

    /**
     * Any time we are paused we need to save away the current state, so it
     * will be restored correctly when we are resumed.
     */
    @Override
    protected void onPause() {
        super.onPause();
        EditText zipText = (EditText) findViewById(R.id.zip_Entry);
        SharedPreferences.Editor editor = getPreferences(0).edit();
        editor.putString("search_text", zipText.getText().toString());
        editor.putInt("selection-start", zipText.getSelectionStart());
        editor.putInt("selection-end", zipText.getSelectionEnd());
        editor.commit();
    }
	
	public String getNiceName() {
		return NICE_NAME;
	}
	
	public String getIconPath() {
    	return "res/drawable/icon_weather2";
    }
	
	public int getIconAddress() {
    	return R.drawable.icon_weather2;
    }
	
	public Class getActivityClass() {
		return WeatherReal.class;
	}
	
}